/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "textbox.h"
#include "assert/assert.h"
#include "input.h"
#include "time.h"

textbox_t TEXTBOX;

void textboxInit() {
  memset(&TEXTBOX, 0, sizeof(textbox_t));
}

void textboxSetText(
  const char_t *title,
  const char_t *text
) {
  assertNotNull(text, "Text cannot be NULL.");

  // Setup text copies
  size_t len = strlen(text);
  assertTrue(len < TEXTBOX_TEXT_MAX, "Text is too long.");
  strcpy(TEXTBOX.text, text);
  TEXTBOX.textLength = len;

  // Setup title
  if(title) {
    len = strlen(title);
    assertTrue(len < TEXTBOX_TITLE_MAX, "Title is too long.");
    strcpy(TEXTBOX.title, title);
  } else {
    TEXTBOX.title[0] = '\0';
  }

  // Reset UI.
  frameUIReset();

  // Add > indicator
  FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 4] = ' ';
  FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 3] = '>';
  FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 2] = ' ';

  // Add title
  len = strlen(TEXTBOX.title);
  for(size_t i = 0; i < len; i++) {
    FRAME_BUFFER[
      (FRAME_HEIGHT - FRAME_BOTTOM_UI_HEIGHT) * (FRAME_WIDTH) + 2 + i
    ] = TEXTBOX.title[i];
  }

  // Prepare for scrolling
  TEXTBOX.nextChar = (1.0f / TEXTBOX_CHARS_PER_SECOND);
  TEXTBOX.textIndex = 0;
  TEXTBOX.open = true;
}

bool_t textboxIsOpen() {
  return TEXTBOX.open;
}

void textboxUpdate() {
  if(!textboxIsOpen()) return;

  int32_t j;

  // Have we finished scrolling?
  if(TEXTBOX.textIndex >= TEXTBOX.textLength) {
    // Wait for input
    if(inputWasPressed(INPUT_BIND_ACCEPT)) {
      TEXTBOX.open = false;
      frameUIReset();
      return;
    }

    // Blinking cursor
    j = (int32_t)(TIME.time * TEXTBOX_BLINKS_PER_SECOND) % 2;
    if(j == 0) {
      FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 3] = '>';
      FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 2] = ' ';
    } else {
      FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 3] = ' ';
      FRAME_BUFFER[(FRAME_HEIGHT * FRAME_WIDTH) - 2] = '>';
    }
    
    return;
  }

  // Update scrolling
  TEXTBOX.nextChar -= TIME.delta;
  if(inputIsDown(INPUT_BIND_ACCEPT)) TEXTBOX.nextChar -= TIME.delta;
  if(TEXTBOX.nextChar > 0.0f) return;
  
  // Scroll text
  uint16_t x = TEXTBOX.textIndex % (FRAME_WIDTH - 2);
  uint16_t y = TEXTBOX.textIndex / (FRAME_WIDTH - 2);
  FRAME_BUFFER[
    (FRAME_HEIGHT - FRAME_BOTTOM_UI_HEIGHT + 1 + y) *
    (FRAME_WIDTH) + 1 + x
  ] = TEXTBOX.text[TEXTBOX.textIndex];
  TEXTBOX.textIndex++;
  TEXTBOX.nextChar = (1.0f / TEXTBOX_CHARS_PER_SECOND);
}